# This is the location of the site_control library. Not needed it you installed
# Apache::SiteControl in your system paths.
use lib qw(/home/tkay/src/site_control2/lib);

# The location of the extra modules used in the sample. Modify to match your
# extract directory
use lib qw(/home/tkay/src/site_control2/sample);

1;
